@@ -37,8 +37,9 @@ class FeaturesToModel(Transform):
         model_import_str=None,
         custom_script=None,
         custom_args=None,
-        inference_arch="x86_64",
+        training_image="training",
         inference_image="inference",
+        inference_arch="x86_64",
     ):
         """FeaturesToModel Initialization
         Args:
@@ -49,8 +50,9 @@ class FeaturesToModel(Transform):
             model_import_str (str, optional): The import string for the model (default None)
             custom_script (str, optional): Custom script to use for the model (default None)
             custom_args (dict, optional): Custom arguments to pass to custom model scripts (default None)
-            inference_arch (str, optional): Inference architecture (default "x86_64")
+            training_image (str, optional): Training image (default "training")
             inference_image (str, optional): Inference image (default "inference")
+            inference_arch (str, optional): Inference architecture (default "x86_64")
         """
 
         # Make sure the model_name is a valid name
@@ -73,8 +75,9 @@ class FeaturesToModel(Transform):
         self.model_feature_list = None
         self.target_column = None
         self.class_labels = None
-        self.inference_arch = inference_arch
+        self.training_image = training_image
         self.inference_image = inference_image
+        self.inference_arch = inference_arch
 
     def transform_impl(
         self, target_column: str, description: str = None, feature_list: list = None, train_all_data=False, **kwargs
@@ -161,12 +164,16 @@ class FeaturesToModel(Transform):
             "model_metrics_s3_path": self.model_training_root,
             "train_all_data": train_all_data,
             "id_column": feature_set.id_column,
+            "hyperparameters": kwargs.get("hyperparameters", {}),
         }
 
         # Custom Script
         if self.custom_script:
             script_path = self.custom_script
             if self.custom_script.endswith(".template"):
+                # Model Type is an enumerated type, so we need to convert it to a string
+                template_params["model_type"] = template_params["model_type"].value
+
                 # Fill in the custom script template with specific parameters (include any custom args)
                 template_params.update(self.custom_args)
                 script_path = fill_template(self.custom_script, template_params, "generated_model_script.py")
@@ -226,7 +233,7 @@ class FeaturesToModel(Transform):
         source_dir = str(Path(script_path).parent)
 
         # Create a Sagemaker Model with our script
-        image = ModelImages.get_image_uri(self.sm_session.boto_region_name, "training", "0.1")
+        image = ModelImages.get_image_uri(self.sm_session.boto_region_name, self.training_image, "0.1")
         self.estimator = Estimator(
             entry_point=entry_point,
             source_dir=source_dir,
@@ -243,6 +250,7 @@ class FeaturesToModel(Transform):
         training_job_name = f"{self.output_name}-{training_date_time_utc}"
 
         # Train the estimator
+        self.log.important(f"Training the Model {self.output_name} with Training Image {image}...")
         self.estimator.fit({"train": s3_training_path}, job_name=training_job_name)
 
         # Now delete the training data
@@ -256,6 +264,11 @@ class FeaturesToModel(Transform):
         self.log.important(f"Creating new model {self.output_name}...")
         self.create_and_register_model(**kwargs)
 
+        # Make a copy of the training view, to lock-in the training data used for this model
+        model_training_view_name = f"{self.output_name.replace('-', '_')}_training"
+        self.log.important(f"Creating Model Training View: {model_training_view_name}...")
+        feature_set.view("training").copy(f"{model_training_view_name}")
+
     def post_transform(self, **kwargs):
         """Post-Transform: Calling onboard() on the Model"""
         self.log.info("Post-Transform: Calling onboard() on the Model...")
@@ -273,11 +286,12 @@ class FeaturesToModel(Transform):
         # Call the Model onboard method
         output_model.onboard_with_args(self.model_type, self.target_column, self.model_feature_list)
 
-    def create_and_register_model(self, aws_region=None):
+    def create_and_register_model(self, aws_region=None, **kwargs):
         """Create and Register the Model
 
         Args:
             aws_region (str, optional): AWS Region to use (default None)
+            **kwargs: Additional keyword arguments to pass to the model registration
         """
 
         # Get the metadata/tags to push into AWS
@@ -294,7 +308,7 @@ class FeaturesToModel(Transform):
         image = ModelImages.get_image_uri(
             self.sm_session.boto_region_name, self.inference_image, "0.1", self.inference_arch
         )
-        self.log.important(f"Registering model {self.output_name} with image {image}...")
+        self.log.important(f"Registering model {self.output_name} with Inference Image {image}...")
         model = self.estimator.create_model(role=self.workbench_role_arn)
         if aws_region:
             self.log.important(f"Setting AWS Region: {aws_region} for model {self.output_name}...")
@@ -316,10 +330,10 @@ if __name__ == "__main__":
 
     # Regression Model
     input_name = "abalone_features"
-    output_name = "abalone-regression"
+    output_name = "test-abalone-regression"
     to_model = FeaturesToModel(input_name, output_name, model_type=ModelType.REGRESSOR)
-    to_model.set_output_tags(["abalone", "public"])
-    to_model.transform(target_column="class_number_of_rings", description="Abalone Regression")
+    to_model.set_output_tags(["test"])
+    to_model.transform(target_column="class_number_of_rings", description="Test Abalone Regression")
 
     """
     # Classification Model
